package matrix.methods;

import java.util.*;

public class Exercise6 {
	
	/** Random initialization of A values 
	 *  @param A Matrix to initialize 
	 *  @param a Lower limit 
	 *  @param b Upper limit (not included) */
	public static void randomInitialization(double [][] A, double a, double b) {
		
		/* Initialize each element of A*/
		for(int i=0; i<A.length; i++)
			for(int j=0; j<A[i].length; j++)
				A[i][j] = a + (b-a)*Math.random();
	}
	
	/** Read matrix values 
	 *  @param A Matrix to store values in */
	public static void readValues(double [][] A) {
		Scanner sc = new Scanner(System.in);
		
		/* Read each element of A */
		for(int i=0; i<A.length; i++)
			for(int j=0; j<A[i].length; j++) {
				boolean readOk = false;
				
				double x = 0;
				while( !readOk ) {
					
					try {
						System.out.print("[" + i + "][" + j + "]: ");
						x = sc.nextDouble();
					} catch(InputMismatchException e) {
						sc.next();
						System.out.println("Wrong number format");
						continue;
					}
					
					readOk = true;
				}
				
				A[i][j] = x;
			}
	}
	
	/** Print input matrix A 
	 *   @param A Matrix to be printed */
	public static void print(double [][] A) {
		
		/* Print each element of A */
		for(int i=0; i<A.length; i++) {
			System.out.println();
			for(int j=0; j<A[i].length; j++)
				System.out.print(A[i][j] + " ");			
		}
		
	}
	
	/** Get the max value 
	 * @param A Array to traverse
	 * @return Largest value of m */
	public static double max(double [][] A) {
		double maxVal = Double.MIN_VALUE;
		
		for(int i=0; i<A.length; i++)
			for(int j=0; j<A[i].length; j++)
				if(maxVal < A[i][j])
					maxVal = A[i][j];
		
		return maxVal;
	}
	
	/** Adds input matrices A and B 
	 *  We assume that A and B are regular matrices correctly initialized
	 *  @param A Matrix 1 
	 *  @param B Matrix 2 
	 *  @return  C = A + B */
	public static double [][] add(double [][] A, double [][] B) {
		
		/* Create result matrix */
		double [][] C = null;
		
		if(A.length == B.length && A[0].length == B[0].length) {
			C = new double[A.length][A[0].length];

			/* Adding A and B */
			for(int i=0; i<A.length; i++)
				for(int j=0; j<A[0].length; j++)
					C[i][j] = A[i][j] + B[i][j];
		}
		
		/* Return result */
		return C;		
	}
	
	/** Subtract input matrices A and B 
	 *  We assume that A and B are regular matrices correctly initialized
	 *  @param A Matrix 1 
	 *  @param B Matrix 2 
	 *  @return  C = A - B */
	public static double [][] sub(double [][] A, double [][] B) {
		
		/* Create result matrix */
		double [][] C = null;
		
		if(A.length == B.length && A[0].length == B[0].length) {
			C = new double[A.length][A[0].length];

			/* Adding A and B */
			for(int i=0; i<A.length; i++)
				for(int j=0; j<A[0].length; j++)
					C[i][j] = A[i][j] - B[i][j];
		}
		
		/* Return result */
		return C;		
	}
	
	/** Multiplication of matrix A by scalar x 
	 *  We assume that A and B are regular matrices correctly initialized
	 *  @param A Matrix 1 
	 *  @return  C = x * A */
	public static double [][] scalar(double x, double [][] A) {
		
		/* Create result matrix */
		double [][] C;
		C = new double[A.length][A[0].length];
		
		/* Adding A and B */
		for(int i=0; i<A.length; i++)
			for(int j=0; j<A[0].length; j++)
				C[i][j] = x * A[i][j];
		
		/* Return result */
		return C;		
	}
	
	/** Multiplication of two REGULAR matrices 
	 * @param a Matrix A 
	 * @param b Matrix B
	 * @result A x B; null if not possible */
	public static double [][] multiplication(double [][] a, double [][] b) {
		double [][] c = null;
		
		if(a[0].length == b.length) {
			// Multiplication is possible
			c = new double[a.length][b[0].length];
			
			for(int i=0; i<c.length; i++)
				for(int j=0; j<c[i].length; j++)
					for(int k=0; k<a[i].length; k++)
						c[i][j] += (a[i][k] * b[k][j]); 
		}
		
		return c;
	}
	
	/** Obtains the transposed of m 
	 * @param m Array to transpose
	 * @return Transposed of m */
	public static double [][] transposed(double [][] m) {
		double [][] t = new double[m[0].length][m.length];
		
		for(int i=0; i<m.length; i++)
			for(int j=0; j<m[i].length; j++)
				t[j][i] = m[i][j];		
		
		return t;
	}	
	
	
//	public static void main(String[] args) {
//		
//		/* Create matrices */
//		double [][] matrix1 = new double[2][3]; // 2x3 matrix
//		double [][] matrix2 = new double[2][3]; // 2x3 matrix
//		double [][] matrix3 = new double[3][2]; // 3x2 matrix
//		
//		/* Initialize values */
//		readValues(matrix1);
//		randomInitialization(matrix2, 0, 10);
//		randomInitialization(matrix3, 0, 10);
//		
//		/* Add matrices */
//		double [][] result;	// result does not need to be initialized
//		result = add(matrix1, matrix2);
//		
//		/* Print result */
//		print(result);
//		
//		/* Print result without storing the value */
//		print(add(matrix1, matrix2));	
//		
//		/* Transposed */
//		double [][] t = transposed(matrix3);
//		print(t);
//		
//		/* Max value */
//		System.out.println("\n\nMax value of matrix 1: " + max(matrix1));
//		
//		/* Multiplication */		
//		double [][] mult = multiplication(matrix1, matrix3);
//		print(mult);
//
//		
//	}
	
	public static void main(String [] args) {
		
		double [][] m, n, r;
		
		m = new double[3][4];
		randomInitialization(m, 0, 1);
		
		n = new double[3][4];
		randomInitialization(n, 0, 1);
		
		r = add(m, n);
		print(r);
	}
}
